searchNodes=[{"ref":"FIQLEx.html","title":"FIQLEx","type":"module","doc":"FIQL (Feed Item Query Language) is a URI-friendly syntax for expressing filters. FIQL looks like this: fiql = &quot;author.age=ge=25;author.name==*Doe&quot; Using this module you will be able to parse a FIQL string and to build a query for any system (SQL, Elasticsearch, etc...) from it. Given a FIQL string like: fiql = &quot;author.age=ge=25;author.name==*Doe&quot; Pass it to the parse/1 or parse1!/1 functions to retrieve an AST of the FIQL string: {:ok, ast} = FIQLEx.parse(fiql) Then you can use this AST to build you own query for your system or use our built-in query builders like FIQLEx.QueryBuilders.SQLQueryBuilder: {:ok, sql_query} = FIQLEx.build_query(ast, FIQLEx.QueryBuilders.SQLQueryBuilder, table: &quot;author&quot;) Here, sql_query is SELECT * FROM author WHERE (author.age &gt;= 25 AND author.name LIKE &#39;%Doe&#39;). You can use your own query builder by providing your own module that uses FIQLEx.QueryBuilder as second argument of build_query/3."},{"ref":"FIQLEx.html#build_query/3","title":"FIQLEx.build_query/3","type":"function","doc":"Use an AST to build a query in the way you want. For instance you could create a query for Elasticsearch from a FIQL AST, or use the FIQLEx.QueryBuilders.SQLQueryBuilder module to build an SQL query from a FIQL AST. Parameters are: ast: The AST to transform to a query for another system module: The module to use for the AST traversal opts: Options you want to pass to the init/2 function of your module This function returns {:ok, query} with your created query if everything is fine, or {:error, reason} if there is something wrong. query = &quot;author.age=ge=25;author.name==*Doe&quot; {:ok, ast} = FIQLEx.parse(query) {:ok, query} = FIQLEx.build_query(ast, MyQueryBuilder) See the documentation of the FIQLEx.QueryBuilder module to learn more about the AST traversal."},{"ref":"FIQLEx.html#handle_ast/4","title":"FIQLEx.handle_ast/4","type":"function","doc":"This function will go deeper in the ast traversal. Parameters are: curr_ast: The AST we want to go deeper with ast: The global AST module: The module to use for the traversal state: The current state of your query builder The function returns {:ok, state} if everything is fine, and {:error, reason} if there is an error"},{"ref":"FIQLEx.html#handle_ast!/4","title":"FIQLEx.handle_ast!/4","type":"function","doc":"Same as handle_ast/4 but returns the state or raises an exception."},{"ref":"FIQLEx.html#parse/1","title":"FIQLEx.parse/1","type":"function","doc":"Parses the FIQL string and returns an AST representation of the query to be built to any other query (SQL, Elasticsearch) with the build_query/3 function. Returns {:ok, ast} if everything is fine and {:error, reason} in case of error in the FIQL."},{"ref":"FIQLEx.html#parse!/1","title":"FIQLEx.parse!/1","type":"function","doc":"Same as parse/1 but returns the AST or raises an exception."},{"ref":"FIQLEx.html#t:ast/0","title":"FIQLEx.ast/0","type":"type","doc":""},{"ref":"FIQLEx.QueryBuilder.html","title":"FIQLEx.QueryBuilder","type":"behaviour","doc":"QueryBuilder module has to be used to build queries from FIQL. You have at least to use this module and define the functions init and build. Here is the minimal code: defmodule MyQueryBuilder do use FIQLEx.QueryBuilder @impl true def init(ast, _opts) do %{} end @impl true def build(_ast, state) do {:ok, state} end end The you&#39;ll want to override functions like handle_and_expression/4, etc, ... to build your final query. See SQLQueryBuilder module for an implementation example. To build a new query using your module you have to parse a FIQL query and call FIQLEx.build_query/3 query = &quot;author.age=ge=25;author.name==*Doe&quot; {:ok, ast} = FIQLEx.parse(query) {:ok, query} = FIQLEx.build_query(ast, MyQueryBuilder)"},{"ref":"FIQLEx.QueryBuilder.html#c:build/2","title":"FIQLEx.QueryBuilder.build/2","type":"callback","doc":"This callback is invoked at the end of the call to the function FIQLEx.build_query/3. Parameters are: ast: The AST returned by FIQLEx.parse/1 state: The current (and final) state of your query builder This function returns {:ok, final_state} if everything is ok, or {:error, reason} if there is something wrong"},{"ref":"FIQLEx.QueryBuilder.html#c:handle_and_expression/4","title":"FIQLEx.QueryBuilder.handle_and_expression/4","type":"callback","doc":"This callback is invoked when an AND expression is found in the query. Parameters are: exp1: left side of the AND expression exp2: right side of the AND expression ast: The AST returned by FIQLEx.parse/1 state: The current state of your query builder This function returns {:ok, new_state} if everything is ok, or {:error, reason} if there is something wrong Example author.age!=25;author.name==*Doe The exp1 is author.age!=25 The exp2 is author.name==*Doe Call handle_ast(exp, ast, state) to go deeper in the expressions"},{"ref":"FIQLEx.QueryBuilder.html#c:handle_expression/3","title":"FIQLEx.QueryBuilder.handle_expression/3","type":"callback","doc":"This callback is invoked when an expression is found in the query. An expression is a selector compared to a value, or just a selector. Parameters are: exp: the expression ast: The AST returned by FIQLEx.parse/1 state: The current state of your query builder This function returns {:ok, new_state} if everything is ok, or {:error, reason} if there is something wrong Example author.age!=25 This is an expression. Call handle_ast(exp, ast, state) to go deeper in the expression"},{"ref":"FIQLEx.QueryBuilder.html#c:handle_or_expression/4","title":"FIQLEx.QueryBuilder.handle_or_expression/4","type":"callback","doc":"This callback is invoked when an OR expression is found in the query. Parameters are: exp1: left side of the OR expression exp2: right side of the OR expression ast: The AST returned by FIQLEx.parse/1 state: The current state of your query builder This function returns {:ok, new_state} if everything is ok, or {:error, reason} if there is something wrong Example author.age!=25,author.name==*Doe The exp1 is author.age!=25 The exp2 is author.name==*Doe Call handle_ast(exp, ast, state) to go deeper in the expressions"},{"ref":"FIQLEx.QueryBuilder.html#c:handle_selector/3","title":"FIQLEx.QueryBuilder.handle_selector/3","type":"callback","doc":"This callback is invoked when a selector without a value to be compared to is found. Parameters are: selector_name: the name of the selector ast: The AST returned by FIQLEx.parse/1 state: The current state of your query builder This function returns {:ok, new_state} if everything is ok, or {:error, reason} if there is something wrong Example author.age The selector_name is author.age"},{"ref":"FIQLEx.QueryBuilder.html#c:handle_selector_and_value/5","title":"FIQLEx.QueryBuilder.handle_selector_and_value/5","type":"callback","doc":"This callback is invoked when a selector is found with a value it is compared to. Parameters are: selector_name: the name of the selector op: the comparison operator. Either :equal or :not_equal value: The value to compare the selector to ast: The AST returned by FIQLEx.parse/1 state: The current state of your query builder This function returns {:ok, new_state} if everything is ok, or {:error, reason} if there is something wrong Example author.age==25 The selector_name is author.age The op is :equal The value is 25"},{"ref":"FIQLEx.QueryBuilder.html#c:handle_selector_and_value_with_comparison/5","title":"FIQLEx.QueryBuilder.handle_selector_and_value_with_comparison/5","type":"callback","doc":"This callback is invoked when a selector is found with a value it is compared to. Same as handle_selector_and_value/5 but with a custom comparison operator Parameters are: selector_name: the name of the selector op: the comparison operator as a string value: The value to compare the selector to ast: The AST returned by FIQLEx.parse/1 state: The current state of your query builder This function returns {:ok, new_state} if everything is ok, or {:error, reason} if there is something wrong Example author.age=ge=25 The selector_name is author.age The op is ge The value is 25"},{"ref":"FIQLEx.QueryBuilder.html#c:init/2","title":"FIQLEx.QueryBuilder.init/2","type":"callback","doc":"This callback is invoked as soon as you call the function FIQLEx.build_query/3. Parameters are: ast: The AST returned by FIQLEx.parse/1 options: The options you passed to FIQLEx.build_query/3 This function must return the initial state of your query builder"},{"ref":"FIQLEx.QueryBuilder.Helpers.html","title":"FIQLEx.QueryBuilder.Helpers","type":"module","doc":""},{"ref":"FIQLEx.QueryBuilder.Helpers.html#do_get_selectors/1","title":"FIQLEx.QueryBuilder.Helpers.do_get_selectors/1","type":"function","doc":""},{"ref":"FIQLEx.QueryBuilder.Helpers.html#do_handle_ast/4","title":"FIQLEx.QueryBuilder.Helpers.do_handle_ast/4","type":"function","doc":""},{"ref":"FIQLEx.QueryBuilder.Helpers.html#do_handle_ast!/4","title":"FIQLEx.QueryBuilder.Helpers.do_handle_ast!/4","type":"function","doc":""},{"ref":"FIQLEx.QueryBuilders.SQLQueryBuilder.html","title":"FIQLEx.QueryBuilders.SQLQueryBuilder","type":"module","doc":"Builds SQL queries from FIQL AST. Possible options for this query builder are: table: The table name to use in the FROM statement (defaults to &quot;table&quot;) select: SELECT statement to build (see below). Select option Possible values of the select option are: :all: use SELECT * (default value) :from_selectors: Searches for all selectors in the FIQL AST and use them as SELECT statement. For instance, for the following query: age=ge=25;name==*Doe, the SELECT statement will be SELECT age, name selectors: You specify a list of items you want to use in the SELECT statement. Examples iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==John&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name = &#39;John&#39;&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==John&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors, table: &quot;author&quot;) {:ok, &quot;SELECT name FROM author WHERE name = &#39;John&#39;&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==John&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :all) {:ok, &quot;SELECT * FROM table WHERE name = &#39;John&#39;&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==John&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: [&quot;another&quot;, &quot;other&quot;]) {:ok, &quot;SELECT another, other FROM table WHERE name = &#39;John&#39;&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==John;(age=gt=25,age=lt=18)&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name, age FROM table WHERE (name = &#39;John&#39; AND (age &gt; 25 OR age &lt; 18))&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name=ge=John&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:error, :invalid_format} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name=ge=2019-02-02T18:23:03Z&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name &gt;= &#39;2019-02-02T18:23:03Z&#39;&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name!=12.4&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name &lt;&gt; 12.4&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name!=true&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name &lt;&gt; true&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name!=false&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name &lt;&gt; false&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name IS NOT NULL&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name!=(1,2,Hello)&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name NOT IN (1, 2, &#39;Hello&#39;)&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name!=&#39;Hello \\\\&#39;World&#39;&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name &lt;&gt; &#39;Hello &#39;&#39;World&#39;&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name!=*Hello&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name NOT LIKE &#39;%Hello&#39;&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==Hello*&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name FROM table WHERE name LIKE &#39;Hello%&#39;&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==Hello;age=ge=10;friend==true&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name, age, friend FROM table WHERE (name = &#39;Hello&#39; AND (age &gt;= 10 AND friend = true))&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==Hello,age=ge=10,friend==true&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name, age, friend FROM table WHERE (name = &#39;Hello&#39; OR (age &gt;= 10 OR friend = true))&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==Hello;age=ge=10;friend==true;ok&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name, age, friend, ok FROM table WHERE (name = &#39;Hello&#39; AND (age &gt;= 10 AND (friend = true AND ok IS NOT NULL)))&quot;} iex&gt; FIQLEx.build_query(FIQLEx.parse!(&quot;name==Hello,age=ge=10,friend==true,ok&quot;), FIQLEx.QueryBuilders.SQLQueryBuilder, select: :from_selectors) {:ok, &quot;SELECT name, age, friend, ok FROM table WHERE (name = &#39;Hello&#39; OR (age &gt;= 10 OR (friend = true OR ok IS NOT NULL)))&quot;}"},{"ref":"FIQLEx.QueryBuilders.SQLQueryBuilder.html#get_selectors/1","title":"FIQLEx.QueryBuilders.SQLQueryBuilder.get_selectors/1","type":"function","doc":"Returns a list of all selectors for a given AST"},{"ref":"FIQLEx.QueryBuilders.SQLQueryBuilder.html#handle_ast/3","title":"FIQLEx.QueryBuilders.SQLQueryBuilder.handle_ast/3","type":"function","doc":"This function will go deeper in the ast traversal. Parameters are: curr_ast: The AST we want to go deeper with ast: The global AST state: The current state of your query builder The function returns {:ok, state} if everything is fine, and {:error, reason} if there is an error"},{"ref":"FIQLEx.QueryBuilders.SQLQueryBuilder.html#handle_ast!/3","title":"FIQLEx.QueryBuilders.SQLQueryBuilder.handle_ast!/3","type":"function","doc":"Same as handle_ast/3 but returns the state or raises an exception."},{"ref":"FIQLEx.SelectorsGetter.html","title":"FIQLEx.SelectorsGetter","type":"module","doc":""},{"ref":"FIQLEx.SelectorsGetter.html#get_selectors/1","title":"FIQLEx.SelectorsGetter.get_selectors/1","type":"function","doc":"Returns a list of all selectors for a given AST"},{"ref":"FIQLEx.SelectorsGetter.html#handle_ast/3","title":"FIQLEx.SelectorsGetter.handle_ast/3","type":"function","doc":"This function will go deeper in the ast traversal. Parameters are: curr_ast: The AST we want to go deeper with ast: The global AST state: The current state of your query builder The function returns {:ok, state} if everything is fine, and {:error, reason} if there is an error"},{"ref":"FIQLEx.SelectorsGetter.html#handle_ast!/3","title":"FIQLEx.SelectorsGetter.handle_ast!/3","type":"function","doc":"Same as handle_ast/3 but returns the state or raises an exception."},{"ref":"readme.html","title":"fiqlex","type":"extras","doc":"fiqlex FIQL (Feed Item Query Language) is a URI-friendly syntax for expressing filters. FIQL looks like this: fiql = &quot;author.age=ge=25;author.name==*Doe&quot; Using this module you will be able to parse a FIQL string and to build a query for any system (SQL, Elasticsearch, etc...) from it."},{"ref":"readme.html#grammar","title":"fiqlex - Grammar","type":"extras","doc":"If you want to know more about FIQL grammar please check the RFC, and my lexer and parser"},{"ref":"readme.html#quick-start","title":"fiqlex - Quick start","type":"extras","doc":"First, add this module to your mix.exs file: defp deps do [ {:fiqlex, &quot;~&gt; 0.1.1&quot;}, ] end Given a FIQL string like: fiql = &quot;author.age=ge=25;author.name==*Doe&quot; Pass it to the parse/1 or parse1!/1 functions to retrieve an AST of the FIQL string: {:ok, ast} = FIQLEx.parse(fiql) Then you can use this AST to build you own query for your system or use our built-in query builders like FIQLEx.QueryBuilders.SQLQueryBuilder: {:ok, sql_query} = FIQLEx.build_query(ast, FIQLEx.QueryBuilders.SQLQueryBuilder, table: &quot;author&quot;) Here, sql_query is SELECT * FROM author WHERE (author.age &gt;= 25 AND author.name LIKE &#39;%Doe&#39;). You can use your own query builder by providing your own module that uses FIQLEx.QueryBuilder as second argument of build_query/3."},{"ref":"readme.html#documentation","title":"fiqlex - Documentation","type":"extras","doc":"Documentation can be found in hexdoc"},{"ref":"readme.html#tests","title":"fiqlex - Tests","type":"extras","doc":"You can run tests with: mix test"}]